;;; -*- lexical-binding:t -*-

(require 'dired)

(defun qv ()
  (let ((config (current-window-configuration))
        (dir-buffer (current-buffer))
        (max-size 1e6)
        (qv-buffer)
        (qv-window))
    (defvar keybinding
      (lambda (&optional arg)
              (interactive)
              (setq-local quick-view-object (qv))
              (apply quick-view-object '(init))
              (message "enter quick view")))
    (defun get-key () keybinding)
    (defun update ()
      (let ((f (dired-get-filename t t)))
        (when (and (stringp f)
                   (file-regular-p f)
                   (< (nth 7 (file-attributes f)) max-size))
          (let ((auto-mode-alist)
                (buf (find-file-noselect f nil t)))
            (when (not (equal buf qv-buffer))
              (and qv-buffer
                   (not (equal (current-buffer) qv-buffer))
                   (kill-buffer qv-buffer))
              (with-selected-window qv-window
                (switch-to-buffer buf)))
            (setq qv-buffer buf)))))
    (defun init ()
      (delete-other-windows)
      (split-window-sensibly)
      (balance-windows)
      (setq qv-window (save-window-excursion (other-window 1)
                                             (selected-window)))
      (add-hook 'post-command-hook 'update 'at-the-end t)
      (define-key dired-mode-map [(control ?c) (control ?q)]
        (lambda (&optional arg)
          (interactive)
          (apply quick-view-object (list 'exit)))))
    (defun exit ()
      (setq-local quick-view-object nil)
      (remove-hook 'post-command-hook 'update t)
      (define-key dired-mode-map [(control ?c) (control ?q)]
        keybinding )
      (and qv-buffer (kill-buffer qv-buffer))
      (set-window-configuration config)
      (message "exit qv"))
    (lambda (message)
      (apply message '()))))

(define-key dired-mode-map [(control ?c) (control ?q)]
  (apply (qv) '(get-key)))

